/* 
   Figment - a Gtk2 based outliner.  

   Copyright 2002 Matthew Allum

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "export.h"

extern Figment Fig;

static void
export_recurse( Exporter *e, GtkTreeIter *iter )
{
  gboolean valid = 1;
  GtkTreeModel *model;
  gchar *thgt_data;
  gint   chk_data;
  gboolean  is_expanded = FALSE;
  GtkTreePath *path;
  GtkTreeIter child_iter;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW(Fig.treeview));

  e->depth++;
  if (e->start_branch_cb) e->start_branch_cb(e, e->userdata);

  while (valid)
    {
       
      gtk_tree_model_get (model, iter, 
                          THOUGHT_COLUMN, &thgt_data,
                          CHECKED_COLUMN, &chk_data,
                          -1);

      path = gtk_tree_model_get_path(model, iter);
      gtk_tree_view_row_expanded(GTK_TREE_VIEW(Fig.treeview), path);

      if (e->start_item_cb)
	e->start_item_cb(e, thgt_data, chk_data, is_expanded, e->userdata);

      g_free (thgt_data);
      gtk_tree_path_free(path);

      if (gtk_tree_model_iter_children(model, &child_iter, iter))
	export_recurse( e, &child_iter );

      if (e->end_item_cb)
	e->end_item_cb(e, thgt_data, chk_data, is_expanded, e->userdata);

      valid = gtk_tree_model_iter_next (model, iter);
      
    }

  if (e->end_branch_cb) e->end_branch_cb(e, e->userdata);
  e->depth--;

}

static void 
export_doit(Exporter *e)
{
  GtkTreeIter iter;
  GtkTreeModel *model;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW(Fig.treeview));

  if (e->header_cb) e->header_cb(e, e->userdata);

  if (gtk_tree_model_get_iter_first(model, &iter))
    export_recurse(e, &iter);

  if (e->footer_cb) e->footer_cb(e, e->userdata);

}

Exporter *
export_export(char *filename,
	    void (*start_item_cb) (
				   struct _exporter *exporter,
				   gchar *thought_data, 
				   gboolean is_checked,
				   gboolean is_expanded,
				   void  *userdata 
				   ),
	    void (*end_item_cb) (
				 struct _exporter *exporter,
				 gchar *thought_data, 
				 gboolean is_checked,
				 gboolean is_expanded,
				 void  *userdata 
				 ),
	    void (*start_branch_cb) (  struct _exporter *exporter, 
				 void  *userdata  ),
	    void (*end_branch_cb) (  struct _exporter *exporter, 
				 void  *userdata  ),

	    void (*header_cb) (  struct _exporter *exporter, 
				 void  *userdata  ),
	    void (*footer_cb) (  struct _exporter *exporter, 
				 void  *userdata  ),

	    void *userdata
	    )
{
  Exporter *e;

  e = (Exporter *)malloc(sizeof(Exporter));

  if ((e->fh = fopen(filename, "w")) == NULL)
    {
      /* TODO: set error */
      free(e);
      return NULL;
    }

  e->start_item_cb = start_item_cb;
  e->end_item_cb   = end_item_cb;
  e->header_cb     = header_cb;
  e->footer_cb     = footer_cb;
  e->start_branch_cb = start_branch_cb;
  e->end_branch_cb = end_branch_cb;

  e->depth = 0;
  e->userdata = userdata;

  export_doit(e);

  fclose(e->fh);

  return e;
}

/* ----- HTML ----- */

static void
export_html_header(Exporter *e, void *userdata)
{
  fprintf(e->fh, "<html><body>\n");
}

static void
export_html_footer(Exporter *e, void *userdata)
{
  fprintf(e->fh, "</body></html>\n");
}

static void
export_html_start_branch(Exporter *e, void  *userdata )
{
    fprintf(e->fh, "<ul>\n");
}

static void
export_html_end_branch(Exporter *e, void  *userdata )
{
  fprintf(e->fh, "</ul>\n");
}


static void
export_html_item_start(Exporter *e,
		   gchar *thought_data, 
		   gboolean is_checked,
		   gboolean is_expanded,
		   void *userdata)
{
  //  if (is_expanded)
    fprintf(e->fh, "<li>%s\n", (char *)thought_data);

}

static void
export_html_item_end(Exporter *e,
		     gchar *thought_data, 
		     gboolean is_checked,
		     gboolean is_expanded,
		     void *userdata)
{
  // if (is_expanded)
    fprintf(e->fh, "</li>\n");
}

int
export_html(char *filename)
{
  Exporter *e;

  e = export_export(filename, 
		    export_html_item_start, 
		    export_html_item_end,
		    export_html_start_branch,
		    export_html_end_branch,
		    export_html_header,
		    export_html_footer,
		    NULL);
  if (e != NULL) return 1;
  return 0;
}

/* -- DocBook -- */

static void
export_docbook_header(Exporter *e, void *userdata)
{
  fprintf(e->fh, 
	  "<!DOCTYPE article PUBLIC \"-//OASIS//DTD DocBook V3.1//EN\">\n"
	  "<article>\n"
	  "<artheader>\n"
	  "<title>Figment Export</title>\n"
	  "</artheader>"
	  );
}

static void
export_docbook_footer(Exporter *e, void *userdata)
{
  fprintf(e->fh, "</article>\n");
}

static void
export_docbook_start_branch(Exporter *e, void  *userdata )
{
    fprintf(e->fh, "<section>\n");
}

static void
export_docbook_end_branch(Exporter *e, void  *userdata )
{
  fprintf(e->fh, "</section>\n");
}


static void
export_docbook_item_start(Exporter *e,
		   gchar *thought_data, 
		   gboolean is_checked,
		   gboolean is_expanded,
		   void *userdata)
{
  //  if (is_expanded)
    fprintf(e->fh, "<title>%s</title>\n", (char *)thought_data);

}

static void
export_docbook_item_end(Exporter *e,
		     gchar *thought_data, 
		     gboolean is_checked,
		     gboolean is_expanded,
		     void *userdata)
{
  // if (is_expanded)
  ;
}

int
export_docbook(char *filename)
{
  Exporter *e;

  e = export_export(filename, 
		    export_docbook_item_start, 
		    export_docbook_item_end,
		    export_docbook_start_branch,
		    export_docbook_end_branch,
		    export_docbook_header,
		    export_docbook_footer,
		    NULL);
  if (e != NULL) return 1;
  return 0;
}

/* Text */

static void
export_text_item_start(Exporter *e,
		   gchar *thought_data, 
		   gboolean is_checked,
		   gboolean is_expanded,
		   void *userdata)
{
  int i;
  for (i = 0; i < e->depth; i++)
    fprintf(e->fh, "\t");
  fprintf(e->fh, "o %s\n", (char *)thought_data);
}

int
export_text(char *filename)
{
  Exporter *e;

  e = export_export(filename, 
		    export_text_item_start, 
		    NULL,
		    NULL,
		    NULL,
		    NULL,
		    NULL,
		    NULL);
  if (e != NULL) return 1;
  return 0;
}
