/* 
   Figment - a Gtk2 based outliner.  

   Copyright 2002 Matthew Allum

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "opml.h"

#define _(_x) gettext (_x)
#define N_(_x) (_x)

static void _opml_save_recurse( xmlNodePtr parent, GtkTreeIter *iter, 
				GList **list );
static GList * opml_parse_head(xmlDocPtr doc, xmlNodePtr cur);
static void
opml_parse_body(xmlDocPtr doc, 
		xmlNodePtr cur, 
		GtkTreeStore *treestore, 
		GtkTreeIter *parent_iter,
		GList *exp_list);

extern Figment Fig;

static void
opml_parse_body(xmlDocPtr doc, 
		xmlNodePtr cur, 
		GtkTreeStore *treestore, 
		GtkTreeIter *parent_iter,
		GList *exp_list) 
{
  GtkTreeIter iter;

  while (cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"outline"))) 
      {
	/*
	printf("appending\n");
	if (parent_iter == NULL)
	  gtk_tree_store_append (treestore, &iter, NULL); 
	else
	*/
	gtk_tree_store_append (treestore, &iter, parent_iter); 

	gtk_tree_store_set (treestore, &iter,
			    CHECKED_COLUMN, 
			    xmlGetProp(cur, "checked") ? TRUE : FALSE,
			    EDITABLE_COLUMN, TRUE,
			    -1);

	figment_set_cell_text(&iter, xmlGetProp(cur, "text") );
	
	if (cur->xmlChildrenNode != NULL)
	  {
	    opml_parse_body(doc, cur->xmlChildrenNode, treestore, 
			    &iter, exp_list);
	  }
      }
    cur = cur->next;
  }
  return;
}

static void
opml_expand_loaded(GtkTreeIter *iter, GList *exp_list)
{
  gboolean valid = 1;
  static int i;
  GtkTreeModel *model;
  GList *list;
  
  model = gtk_tree_view_get_model (GTK_TREE_VIEW(Fig.treeview));
  
  while (valid)
    {
      GtkTreePath *path;
      GtkTreeIter child_iter;
      i++;

      list = g_list_first(exp_list);

      while (list != NULL)
	{
	  if (GPOINTER_TO_INT(list->data) == i)
	    {
	      path = gtk_tree_model_get_path(model, iter);
	      gtk_tree_view_expand_row(GTK_TREE_VIEW(Fig.treeview), 
				       path, FALSE);
	      gtk_tree_path_free(path);
	    }
	  list = g_list_next(list);
	}

      if (gtk_tree_model_iter_children(model, &child_iter, iter))
	{
	  opml_expand_loaded( &child_iter, exp_list );
	}
      valid = gtk_tree_model_iter_next (model, iter);
    }

}

static GList *
opml_parse_head(xmlDocPtr doc, xmlNodePtr cur) 
{
  const char delim[] = ",";
  char *exp_str = NULL;
  GList *list = NULL;
  char *token, *tmp_str;

  while (cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"expansionState"))) 
      {
	exp_str = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
	if (exp_str)
	  {
	    int val;
	    tmp_str = strdup(exp_str);
	    while( (token = strsep (&tmp_str, delim)) != NULL)
	      {
		val = atoi(token);
		list = g_list_append(list, GINT_TO_POINTER(val));
	      }
	  }
      }
    cur = cur->next;
  } 
  return list;
}
      

GtkTreeModel *
opml_load(gchar *filename, GtkTreeStore *treestore)
{
  xmlDocPtr doc;
  xmlNodePtr cur;
  GList *exp_list = NULL;
  GtkTreeIter iter;


  /*
   * build an XML tree from a the file;
   */
  doc = xmlParseFile(filename);
  if (doc == NULL) 
    {
      Fig.lasterror = (gchar *)strdup(_("xml parse failed"));
      return NULL;
    }
  /*
   * Check the document is of the right kind
   */
    
  cur = xmlDocGetRootElement(doc);
  if (cur == NULL) {
    Fig.lasterror = (gchar *)strdup(_("empty document"));
    xmlFreeDoc(doc);
    return NULL;
  }

  if (xmlStrcmp(cur->name, (const xmlChar *) "opml")) {
    Fig.lasterror = (gchar *)strdup(_("document of the wrong type, root node != opml"));
    xmlFreeDoc(doc);
    return NULL;
  }

  if (treestore == NULL)
    {
      treestore = gtk_tree_store_new (NUM_COLUMNS,
				      G_TYPE_STRING,
				      G_TYPE_BOOLEAN,
				      G_TYPE_BOOLEAN);
    } else {
      gtk_tree_store_clear (treestore);
    }

  cur = cur->xmlChildrenNode;

  while (cur != NULL) {

    if ((!xmlStrcmp(cur->name, (const xmlChar *)"head"))
	&& cur->xmlChildrenNode != NULL)
      {
	exp_list = opml_parse_head (doc, cur->xmlChildrenNode);
      }
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"body"))
	&& cur->xmlChildrenNode != NULL) 
      {
	opml_parse_body (doc, cur->xmlChildrenNode, treestore, 
			 NULL, exp_list);
      }

    cur = cur->next;
  }

  if (exp_list)
    {
      GtkTreeModel *model 
	= gtk_tree_view_get_model (GTK_TREE_VIEW(Fig.treeview));
      gtk_tree_model_get_iter_first(model, &iter);
      opml_expand_loaded(&iter, exp_list);
    }

  Fig.filename = (gchar *)strdup((char *)filename);
  gtk_window_set_title (GTK_WINDOW (Fig.window), filename);

  return GTK_TREE_MODEL (treestore);      
}

static void
opml_save_head( xmlNodePtr head_node, GList *list )
{
  char exp_str[1024] = "", tmp[8];
  int i=1;

  while (list != NULL)
    {
      if (GPOINTER_TO_INT(list->data))
	{
	  sprintf(tmp,"%i",i);
	  strcat(exp_str, tmp);
	  strcat(exp_str, ",");
	}
      i++;
      list = g_list_next(list);
    }
  if (strlen(exp_str))
    exp_str[strlen(exp_str)-1] = '\0';
  
  xmlNewTextChild (head_node, NULL, "expansionState", exp_str);

}


static void
_opml_save_recurse( xmlNodePtr parent, GtkTreeIter *iter, GList **list )
{
  gboolean valid = 1;
  GtkTreeModel *model;
  xmlNodePtr cur;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW(Fig.treeview));
  
  while (valid)
    {
      gchar *thgt_data;
      gint   chk_data;
      gint expanded;
      GtkTreePath *path;
      GtkTreeIter child_iter;
      
      gtk_tree_model_get (model, iter, 
                          THOUGHT_COLUMN, &thgt_data,
                          CHECKED_COLUMN, &chk_data,
                          -1);

      path = gtk_tree_model_get_path(model, iter);

      expanded = gtk_tree_view_row_expanded(GTK_TREE_VIEW(Fig.treeview), path);
      *list = g_list_append(*list, 
			    GINT_TO_POINTER (expanded));

      cur = xmlNewChild (parent, NULL, "outline", NULL);
      xmlNewProp (cur, "text", thgt_data);
      if (chk_data) xmlNewProp (cur, "checked", "yes");

      g_free (thgt_data);
      gtk_tree_path_free(path);

      if (gtk_tree_model_iter_children(model, &child_iter, iter))
	_opml_save_recurse( cur, &child_iter, list );
      
      valid = gtk_tree_model_iter_next (model, iter);
    }

}

void
opml_save( gchar *filename )
{
  GtkTreeIter iter;
  GtkTreeModel *model;
  xmlDocPtr doc;
  xmlNodePtr head_node, body_node, root;
  GList *list = NULL;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW(Fig.treeview));

  doc = xmlNewDoc("1.0");

  root = xmlNewDocNode(doc, NULL, "opml", NULL );

  xmlDocSetRootElement(doc, root);

  head_node = xmlNewChild (root, NULL, "head", NULL);
  body_node = xmlNewChild (root, NULL, "body", NULL);

  if (gtk_tree_model_get_iter_first(model, &iter))
    _opml_save_recurse( body_node, &iter, &list );

  opml_save_head( head_node, list);

  /* xmlDocDump( stdout, doc ); */

  xmlSaveFormatFile ((char *)filename, doc, 1);

  g_list_free(list);
  xmlFreeDoc(doc);
}
