/* 
   Figment - a Gtk2 based outliner.  

   Copyright 2002 Matthew Allum

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "util.h"

extern Figment Fig;

int
load_config()
{
  xmlDocPtr doc;
  xmlNodePtr cur;
  char filename[256];

  Fig.render_colors = FALSE;
  Fig.render_toplevel_pretty = FALSE;
  Fig.show_toolbar = TRUE;
  Fig.open_last = FALSE;

  sprintf(filename, "%s/%s", getenv("HOME"), CONF_FILE);

  doc = xmlParseFile(filename);
  if (doc == NULL) 
    {
      return 0;
    }
    
  cur = xmlDocGetRootElement(doc);
  if (cur == NULL) {
    xmlFreeDoc(doc);
    return 0;
  }

  if (xmlStrcmp(cur->name, (const xmlChar *) "figment")) {
    xmlFreeDoc(doc);
    return 0;
  }

  cur = cur->xmlChildrenNode;

  while (cur != NULL) {

    if (!xmlStrcmp(cur->name, (const xmlChar *)"appearence"))
      {
	if (xmlGetProp(cur, "colors")
	    && !xmlStrcmp(xmlGetProp(cur, "colors"), "yes"))
	  Fig.render_colors = TRUE;
	if (xmlGetProp(cur, "toplevel")
	    && !xmlStrcmp(xmlGetProp(cur, "toplevel"), "bold"))
	  Fig.render_toplevel_pretty = TRUE;
      }
    else if (!xmlStrcmp(cur->name, (const xmlChar *)"toolbar"))
      {
	if (xmlGetProp(cur, "visible")
	    && !xmlStrcmp(xmlGetProp(cur, "visible"), "no"))
	    Fig.show_toolbar = FALSE;
      }
    else if (!xmlStrcmp(cur->name, (const xmlChar *)"open"))
      {
	if (xmlGetProp(cur, "last"))
	  {
	    Fig.filename = g_strdup(xmlGetProp(cur, "last"));
	    if (!Fig.filename || strlen(Fig.filename) == 0)
	      {
		Fig.open_last = FALSE;
		if (Fig.filename) free(Fig.filename);
		Fig.filename = NULL;
	      }
	    else Fig.open_last = TRUE;
	  }
      }

    cur = cur->next;
  }

  return 1;
}

void 
save_config()
{
  xmlDocPtr doc;
  xmlNodePtr cur, root;
  char filename[256];

  sprintf(filename, "%s/%s", getenv("HOME"), CONF_FILE);

  doc = xmlNewDoc("1.0");

  root = xmlNewDocNode(doc, NULL, "figment", NULL );

  xmlDocSetRootElement(doc, root);

  cur = xmlNewChild (root, NULL, "appearence", NULL);
  xmlNewProp (cur, "colors", Fig.render_colors ? "yes" : "no");
  xmlNewProp (cur, "toplevel", Fig.render_toplevel_pretty ? "bold" : "plain");

  cur = xmlNewChild (root, NULL, "toolbar", NULL);
  xmlNewProp (cur, "visible", Fig.show_toolbar ? "yes" : "no");

  if (Fig.open_last)
    {
      cur = xmlNewChild (root, NULL, "open", NULL);
      xmlNewProp (cur, "last", Fig.filename );
    }
  
  xmlSaveFormatFile ((char *)filename, doc, 1);
  xmlFreeDoc(doc);

}
